# -*- coding: utf-8 -*-

from .interface import APIMetadata, APIResource, APIResourceList, SpriteResource
from .loaders import *

__all__ = [
    "APIResource",
    "APIMetadata",
    "APIResourceList",
    "SpriteResource",
    "ability",
    "berry",
    "berry_firmness",
    "berry_flavor",
    "characteristic",
    "contest_effect",
    "contest_type",
    "egg_group",
    "encounter_condition",
    "encounter_condition_value",
    "encounter_method",
    "evolution_chain",
    "evolution_trigger",
    "gender",
    "generation",
    "growth_rate",
    "item",
    "item_attribute",
    "item_category",
    "item_fling_effect",
    "item_pocket",
    "language",
    "location",
    "location_area",
    "machine",
    "move",
    "move_ailment",
    "move_battle_style",
    "move_category",
    "move_damage_class",
    "move_learn_method",
    "move_target",
    "nature",
    "pal_park_area",
    "pokeathlon_stat",
    "pokedex",
    "pokemon",
    "pokemon_color",
    "pokemon_form",
    "pokemon_habitat",
    "pokemon_shape",
    "pokemon_species",
    "region",
    "stat",
    "super_contest_effect",
    "type_",
    "version",
    "version_group",
]
